@@ -2,7 +2,7 @@ from dataclasses import dataclass
 from typing import Optional, Tuple
 
 from pytoolconfig import UniversalKey, field
-from pytoolconfig.documentation import _type_to_str, _write_model
+from pytoolconfig.documentation import _generate_table, _type_to_str
 
 
 @dataclass
@@ -30,10 +30,10 @@ def test_type_to_str():
 
 
 def test_documentation():
-    lines = list(_write_model(NestedModel))
+    lines = list(_generate_table(NestedModel))
     assert "description" in lines[1]
     assert "foo_other" in lines[3]
     assert "Tool One" in lines[3]
     assert "no" in lines[3]
     assert "str" in lines[3]
-    assert "bool" in lines[5]
+    assert "bool" in lines[6]
